#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use FindBin;
use File::Slurp;
use File::stat;
use List::Util 'max','min';
use Image::Base::GD;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()
use IPC::Run;

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;
my $scale = 2;

my $version = 0;
foreach my $arg (@ARGV) {
  if ($arg =~ /^VERSION=(.*)/) {
    $version = $1;
  }
}

my $program_filename = File::Spec->catfile($FindBin::Bin,
                                           $FindBin::Script);
my $filename = File::Spec->catfile($FindBin::Bin,
                                   File::Spec->updir,
                                   'favicon.png');

my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
print "N= $n_lo to $n_hi\n";
my (@x,@y);
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  $x[$n] = $x;
  $y[$n] = $y;
}

my $size = 16;
my $x_base = $size - $scale - 13;
my $y_base = $size - 2*$scale - 11;

my $image = Image::Base::GD->new (-height => $size,
                                  -width  => $size);
$image->rectangle(0,0, $size-1,$size-1, 'black');
foreach my $n ($n_lo .. $n_hi-1) {
  my $x1 = $x[$n];
  my $y1 = $y[$n];
  my $x2 = $x[$n+1];
  my $y2 = $y[$n+1];
  $x1 *= $scale;
  $y1 *= $scale;
  $x2 *= $scale;
  $y2 *= $scale;
  $x1 += $x_base;
  $y1 += $y_base;
  $x2 += $x_base;
  $y2 += $y_base;
  $y1 = $size-1 - $y1;
  $y2 = $size-1 - $y2;
  ### line: "$x1,$y1 to $x2,$y2"
  $image->line($x1,$y1, $x2,$y2, 'white');
}
$image->save($filename);

# PNG spec 11.3.4.2 suggests RFC822 (or rather RFC1123) for CreationTime
use constant STRFTIME_FORMAT_RFC822 => '%a, %d %b %Y %H:%M:%S %z';
my $stat = File::stat::stat($program_filename)
  or die "Oops, cannot stat $program_filename: $!";
my $creation_time = POSIX::strftime(STRFTIME_FORMAT_RFC822,
                                    localtime($stat->mtime));

my $copyright = File::Slurp::read_file($program_filename);
$copyright =~ /^# (Copyright.*)/m or die;
$copyright = $1;

pngtextadd($filename, 'Author',    'Kevin Ryde');
pngtextadd($filename, 'Generator', "tools/$FindBin::Script");
pngtextadd($filename, 'Title',     'Iterations of the Alternate Paperfolding Curve - icon');
pngtextadd($filename, 'Creation Time', $creation_time);
pngtextadd($filename, 'Copyright', $copyright);
pngtextadd($filename, 'Homepage',  'http://user42.tuxfamily.org/alternate/index.html');
# pngtextadd($filename, 'Version',   $version);

print "uncompressed ",-s $filename,"\n";
# IPC::Run::run(['optipng','-quiet','-o5',$filename]);
# print "compressed   ",-s $filename,"\n";

# IPC::Run::run(['xzgv',$filename]);

exit 0;

# add text to the png image in $filename
sub pngtextadd {
  my ($filename, $keyword, $value) = @_;
  system('pngtextadd', "--keyword=$keyword", "--text=$value", $filename) == 0
    or die "system(pngtextadd)";
}
