#!/usr/bin/perl -w

# Copyright 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph::Maker::TwinAlternateAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'xy_rotate_plus90','xy_rotate_minus90';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 8;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;

my @count;
foreach my $k (0 .. $max_level) {
  my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                 level => $k,
                                 undirected => 1,
                                );
  foreach my $edge ($graph->edges) {
    ### $edge
    my @degrees = sort map {$graph->vertex_degree($_)} @$edge;
    my $type = join(',', @degrees);
    ### $type
    $count[$k]->{$type}++;
  }
}
foreach my $type ('1,1', '1,2', '2,2', '1,3', '2,3', '3,3') {
  my $varnum = $type;
  $varnum =~ s/,//;
  print "% GP-DEFINE  ${var}_${varnum}_samples = [";
  my $join = '';
  foreach my $k (0 .. $#count) {
    my $value = delete $count[$k]->{$type} || 0;
    print $join,$value;
    $join = ',';
  }
  print "];\n";
}
foreach my $k (0 .. $#count) {
  my $href = $count[$k];
  if (%$href) {
    print "Oops, k=$k leftover keys: ",join(' ',keys %$href),"\n";
    exit 1;
  }
}

print "% end generated\n";
exit 0;
