#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Graph::Maker::TwinAlternateAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'TAVertexToZ_xy';
use MyGraphs;
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                               level => $level,
                               undirected => 1);

my $other_segments = MyLineSegments->new;
foreach my $edge ($graph->edges) {
  $other_segments->add(map {TAVertexToZ_xy($_)} @$edge);
}
my $diameter_segments = MyLineSegments->new;

my @diameter_path = MyGraphs::Graph_tree_diameter_path($graph);
foreach my $i (0 .. $#diameter_path-1) {
  my @coords = map {TAVertexToZ_xy($_)} @diameter_path[$i,$i+1];
  $other_segments->remove(@coords);
  $diameter_segments->add(@coords);
}

$Printwrap = 2;
printwrap("\\draw[my diameter] ");
$Printwrap += 2;
$diameter_segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my other] ");
$Printwrap += 2;
$other_segments->printwrap;
printwrap(";\n");

my $diameter = $#diameter_path;
print "  \\newcommand{\\MyDiameter}{$diameter}\n";
print "% GP-Test-Last  TAdiameter($level) == $diameter\n";
print "  % end generated\n";
exit 0;
