#!/usr/bin/perl -w

# Copyright 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Graph::Maker::TwinAlternateAreaTree;
use List::Util 'max';

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'xy_rotate_minus90','xy_rotate_plus90';
use MyGraphs;
use MyLineSegments;
use MyPrintwrap ':all';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 8;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";

sub Graph_vertex_eccentricity_by_sptg {
  my ($graph,$v) = @_;
  my $sptg = $graph->SPT_Dijkstra($v);
  return max(0, 
             map {$sptg->get_vertex_attribute($_,'weight')//0} $sptg->vertices);
}

my @TAdiameter;
my @TAdiameter_n1;
my @TAdiameter_n2;
my @TAheight;
my @TAcornerEcc;
my @TAdiameter_vertices_lists;
foreach my $level (0 .. $max_level) {
  ### $level
  my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                 level => $level,
                                 undirected => 1);
  my @vertices = MyGraphs::Graph_tree_diameter_path($graph);
  if ($vertices[0] > $vertices[-1]) {
    # first vertex smaller for consistency
    @vertices = reverse @vertices; 
  }
  @vertices >= 1 or die;
  my $diameter = $#vertices;
  my ($TAdiameter_n1, $TAdiameter_n2)
    = sort {$a<=>$b} $vertices[0], $vertices[-1];
  push @TAdiameter, $diameter;
  push @TAdiameter_n1, $TAdiameter_n1;
  push @TAdiameter_n2, $TAdiameter_n2;
  push @TAdiameter_vertices_lists, [@vertices];

  push @TAheight, Graph_vertex_eccentricity_by_sptg($graph,0);

  my $c1 = 1 << ($level-1);
  my $ecc1 = Graph_vertex_eccentricity_by_sptg($graph,$c1);
  if ($level >= 1) {
    my $c2 = $c1 - 1;
    my $ecc2 = Graph_vertex_eccentricity_by_sptg($graph,$c2);
    $ecc1 == $ecc2
      or warn "level=$level  c1=$c1 ecc1=$ecc1   c2=$c2 ecc2=$ecc2";
  }
  push @TAcornerEcc, $ecc1;
}

printwrap_indent("% GP-DEFINE  ");

foreach my $elem (['TAdiameter',\@TAdiameter],
                  ['TAdiameter_v1_n',\@TAdiameter_n1],
                  ['TAdiameter_v2_n',\@TAdiameter_n2],
                  ['TAcornerEcc',\@TAcornerEcc],
                  ['TAheight',\@TAheight]) {
  my ($name,$aref) = @$elem;
  $Printwrap = 0;
  printwrap("${name}_generated_samples = {[");
  $Printwrap += 2;
  foreach my $i (0 .. $#$aref) {
    my $value = $aref->[$i];
    printwrap($value . ($i<$#$aref?',':''));
  }
  $Printwrap = 0;
  printwrap("]};\n");
}

$Printwrap = 0;
printwrap("TAdiameter_vertices_generated_samples = {[\n");
$Printwrap += 2;
foreach my $i (0 .. $#TAdiameter_vertices_lists) {
  my $aref = $TAdiameter_vertices_lists[$i];
  printwrap("[");
  foreach my $i (0 .. $#$aref) {
    my $value = $aref->[$i];
    printwrap($value . ($i<$#$aref?',':''));
  }
  printwrap("]" . ($i<$#TAdiameter_vertices_lists?',':'') . "\n");
}
$Printwrap = 0;
printwrap("]};\n");

print "% end generated\n";
exit 0;
