#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;
if (@ARGV) { $level = shift @ARGV; }

print "    % generated by tools/$FindBin::Script $level\n";
my $path = Math::PlanePath::AlternatePaper->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 4;
printwrap("\\node (start) at (0,0) [my dot];\n");
printwrap("\\node (end)   at ($x_hi,$y_hi) [my dot];\n");

my $alternate_segments = MyLineSegments->new;
my $path_segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $alternate_segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my $Diameter = 0;
{
  # shortest path

  my $north = [0,1];
  my $south = [0,-1];
  my $west = [-1,0];
  my $east = [1,0];

  my $x = 0;
  my $y = 0;

  my $move = sub {
    my ($d, $nodraw) = @_;
    my ($dx,$dy) = @$d;
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    if (! $alternate_segments->is_traversed($x,$y, $x2,$y2)) {
      die "oops xy=$x,$y no segment to dxdy=$dx,$dy";
    }
    $alternate_segments->remove($x,$y, $x2,$y2);
    $path_segments->add($x,$y, $x2,$y2);
    # unless ($nodraw) {
    # }
    $x = $x2;
    $y = $y2;
    $Diameter += abs($dx) + abs($dy);
  };

  $move->($east);
  $move->($north);
  if (($level & 1) == 0) {    # k even
    foreach (1 .. 2**($level/2)-2) {
      $move->($east);
      $move->($north);
    }
    $move->($east);
  } else {                    # k odd
    foreach (1 .. 2**(($level+1)/2)-2) {
      $move->($east);
    }
    $move->($south);
  }

  unless ($x==$x_hi && $y==$y_hi) {
    printwrap("\\node (diameter) at ($x,$y) [my dot];\n");
  }
}

foreach my $elem ([$alternate_segments,'my alternate'],
                  [$path_segments,'my path']) {
  my ($segments, $colour) = @$elem;
  $Printwrap = 4;
  printwrap("\\draw[$colour] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

print "    \\newcommand{\\MyLevel}{$level}\n";
print "    \\newcommand{\\MyDiameter}{$Diameter}\n";
print "% GP-Test  Diameter($level) == $Diameter\n";
print "    % end generated\n";
exit 0;
