#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'mul','div','chamfer','xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternatePaper->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $n_start = 3*$n_hi;
my $n_end = $n_start + 4*$n_hi;
my $n_origin = $n_start + $n_hi;
my ($x_origin,$y_origin) = $path->n_to_xy($n_origin);
### $n_start
### $x_origin
### $y_origin

my $xyxy_is_twinalternate_touch = sub {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$n_start <= $_ && $_ < $n_end} @n_list;
  return @n_list == 2;
};

my $transform = sub {
  my ($x,$y) = @_;
  $x -= $x_origin;
  $y -= $y_origin;
  return ($x,$y);
};

$Printwrap = 2;
{
  my ($x,$y) = $path->n_to_xy($n_origin);
  ($x,$y) = $transform->($x,$y);
  printwrap("\\node at ($x,$y) [my dot,name=start];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_origin + 2*$n_hi);
  ($x,$y) = $transform->($x,$y);
  printwrap("\\node at ($x,$y) [my dot,name=end];\n");
}


# twin alternate drawing
my $segments = MyLineSegments->new;
my ($prev_cx, $prev_cy);
$Printwrap = 2;
printwrap("\\draw[my alternate] ");
my $join = '';
$Printwrap = 4;
foreach my $n ($n_start .. $n_end-1) {

  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my $touch_1 = $xyxy_is_twinalternate_touch->($x,$y);
  my $touch_2 = $xyxy_is_twinalternate_touch->($x+$dx,$y+$dy);
  ($x,$y) = $transform->($x,$y);
  ### at: "n=$n  $x,$y  dxdy=$dx,$dy"

  {
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    my $cx = 2*$x + $dx + $lx;
    my $cy = 2*$y + $dy + $ly;
    ($cx,$cy) = mul($cx,$cy);
    if (defined $prev_cx) {
      ### tree segment: "$prev_cx,$prev_cy to $cx,$cy"
      $segments->add($prev_cx,$prev_cy, $cx,$cy);
    }
    ($prev_cx,$prev_cy) = ($cx,$cy);
  }

  if ($touch_1) {
    my $sx = $x . chamfer($dx);
    my $sy = $y . chamfer($dy);
    printwrap("$join($sx,$sy)");
  } else {
    printwrap("$join($x,$y)");
  }
  $join = '--';

  $x += $dx;
  $y += $dy;
  if ($touch_2) {
    $x .= chamfer(-$dx);
    $y .= chamfer(-$dy);
    printwrap("$join($x,$y)");
    $join = '--';
  }
}
printwrap("--cycle;\n");

$Printwrap = 2;
printwrap("\\draw[my tree] ");
$Printwrap += 2;
$segments->printwrap(transform => sub {
                       my ($x,$y) = @_;
                       ($x,$y) = div($x,$y);
                       $x /= 4;
                       $y /= 4;
                       return ($x,$y);
                     });
printwrap(";\n");

print "  % end generated\n";
exit 0;
